/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Gvc = imports.gi.Gvc;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

/*
    PanelManager - Volume
 */
const StatusArea = Main.panel.statusArea || Main.panel._statusArea;

const KEY_SHOW_STATUSAREA = 'show-statusarea';
const KEY_SHOW_VOLUME = 'show-volume';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;
const PANELMENU_BUTTON_NONE = 2;

const STATUSAREA_DEFAULT = 0;
const STATUSAREA_ADVANCED = 1;
const STATUSAREA_ALTERNATIVE = 2;

const PopupStreamSwitchMenuItem = new Lang.Class({
    Name: 'PopupStreamSwitchMenuItem',
    Extends: PopupMenu.PopupSwitchMenuItem,

    _init: function(control, id) {
        this._control = control;
        this._stream = this._control.lookup_stream_id(id);
        if (StatusArea.aggregateMenu) {
            this.parent(PanelTools.getStringCut(this._stream.description, 30), !this._stream.is_muted);
        } else {
            this.parent(this._stream.description, !this._stream.is_muted);
        }
        this.connect('toggled', Lang.bind(this, function() {
            this._control.lookup_stream_id(id).change_is_muted(!this._stream.is_muted);
        }));
        this._connectStream = this._stream.connect('notify::is-muted', Lang.bind(this, function() {
            this.setToggleState(!this._stream.is_muted);
        }));
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._stream.disconnect(this._connectStream);
        }));
    },

    _setDefault: function() {
        if (this.setOrnament) this.setOrnament(PopupMenu.Ornament.CHECK);
        if (this.setShowDot) this.setShowDot(true);
    }
});

const PopupInputMenuItem = new Lang.Class({
    Name: 'PopupInputMenuItem',
    Extends: PopupStreamSwitchMenuItem,

    _init: function(control, id) {
        this.parent(control, id);
        this.connect('activate', Lang.bind(this, function(actor, event) {
            if (this._stream.id !== this._control.get_default_source().id) {
                this._control.set_default_source(this._stream);
                this._stream.change_is_muted(false);
                this.setToggleState(true);
                this.parent(event);
            }
        }));
        if (this._stream.id === this._control.get_default_source().id) this._setDefault();
    }
});

const PopupOutputMenuItem = new Lang.Class({
    Name: 'PopupOutputMenuItem',
    Extends: PopupStreamSwitchMenuItem,

    _init: function(control, id) {
        this.parent(control, id);
        this.connect('activate', Lang.bind(this, function(actor, event) {
            if (this._stream.id !== this._control.get_default_sink().id) {
                this._control.set_default_sink(this._stream);
                this._stream.change_is_muted(false);
                this.setToggleState(true);
                this.parent(event);
            }
        }));
        if (this._stream.id === this._control.get_default_sink().id) this._setDefault();
    }
});

const Volume = new Lang.Class({
    Name: 'Volume',

    _init: function() {
        this._createIndicators();
        this._createSubMenu();
        this._control = StatusArea.aggregateMenu ? StatusArea.aggregateMenu._volume._control : StatusArea.volume._control;
        this._volumeMax = this._control.get_vol_max_norm();
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._onControlStateChanged)),
            this._settings.connect('changed::'+KEY_SHOW_VOLUME, Lang.bind(this, this._onControlStateChanged))
        ];
        this._onControlStateChanged();
        this._connectControl = [
            this._control.connect('state-changed', Lang.bind(this, this._onControlStateChanged)),
            this._control.connect('default-sink-changed', Lang.bind(this, this._readOutput)),
            this._control.connect('default-source-changed', Lang.bind(this, this._readInput)),
            this._control.connect('stream-added', Lang.bind(this, this._maybeShowInput)),
            this._control.connect('stream-removed', Lang.bind(this, this._maybeShowInput))
        ];
    },

    _createIndicators: function() {
        this._inputIndicator = new St.Icon({ icon_name: 'microphone-sensitivity-muted-symbolic', style_class: 'system-status-icon' });
        if (Convenience.checkGSV(3, 4)) {
            this._outputIndicator = StatusArea.volume.actor.get_children()[0];
            StatusArea.volume.actor.remove_actor(this._outputIndicator);
            this._indicators = new St.BoxLayout({ style_class: 'panelmanager-volume-indicators' });
            this._indicators.add(this._outputIndicator);
            this._indicators.add(this._inputIndicator);
            StatusArea.volume.actor.add_actor(this._indicators);
        } else if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
            StatusArea.volume._box.add_actor(this._inputIndicator);
        } else if (Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12) || Convenience.checkGSV(3, 14) || Convenience.checkGSV(3, 16)) {
            StatusArea.aggregateMenu._volume.indicators.add_child(this._inputIndicator);
        }
    },

    _createSubMenu: function() {
        this._outputStreams = new PopupMenu.PopupMenuSection();
        this._inputStreams = new PopupMenu.PopupMenuSection();
        if (Convenience.checkGSV(3, 4)) {
            this._outputTitle = StatusArea.volume._outputTitle;
            this._outputSlider = StatusArea.volume._outputSlider;
            this._inputTitle = StatusArea.volume._inputTitle;
            this._inputSlider = StatusArea.volume._inputSlider;
            let menuItems = StatusArea.volume.menu._getMenuItems();
            for (let i = menuItems.length - 1; i >= 0; i--) {
                if (menuItems[i] === StatusArea.volume._outputSlider) StatusArea.volume.menu.addMenuItem(this._outputStreams, i + 1);
                if (menuItems[i] === StatusArea.volume._inputSlider) StatusArea.volume.menu.addMenuItem(this._inputStreams, i + 1);
            }
        } else if (Convenience.checkGSV(3, 6)) {
            this._outputTitle = StatusArea.volume._volumeMenu._outputTitle;
            this._outputSlider = StatusArea.volume._volumeMenu._outputSlider;
            this._inputTitle = StatusArea.volume._volumeMenu._inputTitle;
            this._inputSlider = StatusArea.volume._volumeMenu._inputSlider;
            let menuItems = StatusArea.volume._volumeMenu._getMenuItems();
            for (let i = menuItems.length - 1; i >= 0; i--) {
                if (menuItems[i] === StatusArea.volume._volumeMenu._outputSlider) StatusArea.volume._volumeMenu.addMenuItem(this._outputStreams, i + 1);
                if (menuItems[i] === StatusArea.volume._volumeMenu._inputSlider) StatusArea.volume._volumeMenu.addMenuItem(this._inputStreams, i + 1);
            }
        } else if (Convenience.checkGSV(3, 8)) {
            this._outputTitle = StatusArea.volume._volumeMenu._output._title;
            this._outputSlider = StatusArea.volume._volumeMenu._output._slider;
            this._inputTitle = StatusArea.volume._volumeMenu._input._title;
            this._inputSlider = StatusArea.volume._volumeMenu._input._slider;
            StatusArea.volume._volumeMenu._output.item.addMenuItem(this._outputStreams);
            StatusArea.volume._volumeMenu._input.item.addMenuItem(this._inputStreams);
        } else if (Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12) || Convenience.checkGSV(3, 14) || Convenience.checkGSV(3, 16)) {
            let menuItems = StatusArea.aggregateMenu._volume._volumeMenu._getMenuItems();
            for (let i = menuItems.length - 1; i >= 0; i--) {
                if (menuItems[i] === StatusArea.aggregateMenu._volume._volumeMenu._output.item) StatusArea.aggregateMenu._volume._volumeMenu.addMenuItem(this._outputStreams, i + 1);
                if (menuItems[i] === StatusArea.aggregateMenu._volume._volumeMenu._input.item) StatusArea.aggregateMenu._volume._volumeMenu.addMenuItem(this._inputStreams, i + 1);
            }
        }
        if (this._outputSlider) this._setSliderIcon(this._outputSlider, new St.Icon({ icon_name: 'audio-volume-high-symbolic', style_class: 'popup-menu-icon' }));
        if (this._inputSlider) this._setSliderIcon(this._inputSlider, new St.Icon({ icon_name: 'audio-input-microphone-symbolic', style_class: 'popup-menu-icon' }));
    },

    _maybeShowInput: function() {
        let showInput = false;
        let recordingApps = this._control.get_source_outputs();
        if (this._input && recordingApps) {
            for (let i = 0; i < recordingApps.length; i++) {
                let outputStream = recordingApps[i];
                let id = outputStream.get_application_id();
                if (!id || (id !== 'org.gnome.VolumeControl' && id !== 'org.PulseAudio.pavucontrol')) {
                    showInput = true;
                    break;
                }
            }
        }
        let defaultStatusArea = this._settings.get_enum(KEY_SHOW_STATUSAREA) === STATUSAREA_DEFAULT;
        this._inputIndicator.visible = showInput && !defaultStatusArea;
        if (this._inputTitle) this._inputTitle.actor.visible = showInput && defaultStatusArea;
        if (this._inputSlider) this._inputSlider.icon.visible = showInput && !defaultStatusArea;
        this._inputStreams.actor.visible = showInput && !defaultStatusArea && this._inputStreams.numMenuItems > 0;
    },

    _mutedChanged: function(object, param_spec, property) {
        let muted = this[property].is_muted;
        if (property === '_input') {
            if (muted) {
                this._inputIndicator.icon_name = 'microphone-sensitivity-muted-symbolic';
            } else {
                this._inputIndicator.icon_name = this._volumeToIcon(this._input.volume);
            }
        }
    },

    _onControlStateChanged: function() {
        if (this._control.get_state() === Gvc.MixerControlState.READY) {
            this._readOutput();
            this._readInput();
        }
        let showVolume = this._settings.get_enum(KEY_SHOW_VOLUME);
        if (showVolume !== PANELMENU_BUTTON_DEFAULT) {
            let visible = showVolume === PANELMENU_BUTTON_SHOW;
            if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
               StatusArea.volume.actor.visible = visible;
            } else if (Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12) || Convenience.checkGSV(3, 14) || Convenience.checkGSV(3, 16)) {
                StatusArea.aggregateMenu._volume.indicators.visible = visible;
                StatusArea.aggregateMenu._volume.menu.actor.visible = visible;
            }
        }
    },

    _readInput: function() {
        this._inputStreams.removeAll();
        if (this._connectInput) for (let i = 0; i < this._connectInput.length; i++) {
            this._input.disconnect(this._connectInput[i]);
        }
        this._input = this._control.get_default_source();
        if (this._input) {
            this._connectInput = [
                this._input.connect('notify::is-muted', Lang.bind(this, this._mutedChanged, '_input')),
                this._input.connect('notify::volume', Lang.bind(this, this._volumeChanged, '_input'))
            ];
            this._mutedChanged(null, null, '_input');
            this._volumeChanged(null, null, '_input');
            let sources = this._control.get_sources();
            if (sources.length > 0) for (let i = 0; i < sources.length; i++) {
                this._inputStreams.addMenuItem(new PopupInputMenuItem(this._control, sources[i].id));
            }
        } else {
            this._inputIndicator.icon_name = 'microphone-sensitivity-muted-symbolic';
        }
        this._maybeShowInput();
    },

    _readOutput: function() {
        this._outputStreams.removeAll();
        this._output = this._control.get_default_sink();
        if (this._output) {
            let sinks = this._control.get_sinks();
            if (sinks.length > 0) for (let i = 0; i < sinks.length; i++) {
                this._outputStreams.addMenuItem(new PopupOutputMenuItem(this._control, sinks[i].id));
            }
        }
        let defaultStatusArea = this._settings.get_enum(KEY_SHOW_STATUSAREA) === STATUSAREA_DEFAULT;
        if (this._outputTitle) this._outputTitle.actor.visible = defaultStatusArea;
        if (this._outputSlider) this._outputSlider.icon.visible = !defaultStatusArea;
        this._outputStreams.actor.visible = !defaultStatusArea && this._outputStreams.numMenuItems > 0;
    },

    _setSliderIcon: function(item, icon) {
        if (icon) {
            item.removeActor(item._slider);
            item._hbox = new St.BoxLayout({ style_class: 'paneltools-popupmenu-box' });
            item.addActor(item._hbox, { span: -1, expand: true });
            item.icon = icon;
            item._hbox.add(item.icon);
            item._hbox.add(item._slider, { expand: true });
        } else if (item.icon) {
            item.removeActor(item._hbox);
            item._hbox.remove_actor(item._slider);
            delete item._hbox;
            delete item.icon;
            item.addActor(item._slider, { span: -1, expand: true });
        }
    },

    _volumeChanged: function(object, param_spec, property) {
        if (property === '_input' && !this._input.is_muted) this._inputIndicator.icon_name = this._volumeToIcon(this._input.volume);
    },

    _volumeToIcon: function(volume) {
        if (volume <= 0) {
            return 'microphone-sensitivity-muted-symbolic';
        } else {
            let n = Math.floor(3 * volume / this._volumeMax) + 1;
            if (n < 2) return 'microphone-sensitivity-low-symbolic';
            if (n >= 3) return 'microphone-sensitivity-high-symbolic';
            return 'microphone-sensitivity-medium-symbolic';
        }
    },


    _restoreIndicators: function() {
        if (Convenience.checkGSV(3, 4)) {
            this._indicators.remove_actor(this._outputIndicator);
            StatusArea.volume.actor.remove_actor(this._indicators);
            StatusArea.volume.actor.add_actor(this._outputIndicator);
        }
        this._inputIndicator.destroy();
    },

    _restoreSubMenu: function() {
        this._inputStreams.destroy();
        this._outputStreams.destroy();
        if (this._inputSlider) this._setSliderIcon(this._inputSlider);
        if (this._inputTitle) this._inputTitle.actor.visible = this._inputSlider.actor.visible;
        if (this._outputSlider) this._setSliderIcon(this._outputSlider);
        if (this._outputTitle) this._outputTitle.actor.show();
    },

    destroy: function() {
        for each(let connection in this._connectControl) {
            this._control.disconnect(connection);
        }
        for each(let connection in this._connectSettings) {
            this._settings.disconnect(connection);
        }
        this._restoreSubMenu();
        this._restoreIndicators();
    }
});

function getExtension() {
    return new Volume();
}
