/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;

const ExtensionUtils = imports.misc.extensionUtils;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    Temperature - Preferences
 */
const KEY_DISPLAY_TEMPERATURE = 'display-temperature';
const KEY_REFRESH_INTERVAL = 'refresh-interval';
const KEY_SHOW_CPU_CORE_TEMPERATURE = 'show-cpu-core-temperature';
const KEY_SHOW_DISK_TEMPERATURE = 'show-disk-temperature';
const KEY_SHOW_FAHRENHEIT = 'show-fahrenheit';
const KEY_SHOW_GRAPHICS_TEMPERATURE = 'show-graphics-temperature';
const KEY_SHOW_SETTINGS = 'show-settings';

const SETTINGS_GLOBAL = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_FAHRENHEIT,
        label:   _("View Fahrenheit"),
        tooltip: _("If true, display the temperature in Fahrenheit instead of Celsius.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_REFRESH_INTERVAL,
        label:   _("Refresh interval"),
        tooltip: _("Select the desired refresh interval in seconds."),
        values:  { '10':  '10',
                   '15':  '15',
                   '30':  '30',
                   '60':  '60',
                   '120': '120',
                   '300': '300',
                   '600': '600' }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_DISPLAY_TEMPERATURE,
        label:   _("Display Temperature"),
        tooltip: _("Select the temperature that displayed in the GNOME Shell panel."),
        values:  { 'cpu':       _("CPU"),
                   'cpu-cores': _("CPU-Core"),
                   'vga':       _("Graphics Card"),
                   'cpu-vga':   _("CPU / Graphics Card") }
    }
];

const SETTINGS_DISPLAY = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_CPU_CORE_TEMPERATURE,
        label:   _("Show CPU Core"),
        tooltip: _("Select the display mode for CPU Core temperature."),
        values:  { 'none':     _("none"),
                   'current':  _("current"),
                   'critical': _("current / critical"),
                   'maximum':  _("current / maximum"),
                   'all':      _("current / maximum / critical") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_DISK_TEMPERATURE,
        label:   _("Show Disk"),
        tooltip: _("If true, display temperature for Disk.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_GRAPHICS_TEMPERATURE,
        label:   _("Show Graphics Card"),
        tooltip: _("Select the display mode for Graphics Card temperature."),
        values:  { 'none':     _("none"),
                   'current':  _("current"),
                   'critical': _("current / critical"),
                   'maximum':  _("current / maximum"),
                   'all':      _("current / maximum / critical") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_SETTINGS,
        label:   _("Show Settings"),
        tooltip: _("If true, display Settings for Temperature.")
    }
];

const Preferences = new GObject.Class({
    Name: 'TemperaturePreferences',
    GTypeName: 'TemperaturePreferences',
    Extends: Gtk.Box,

    _init: function(params) {
        this.parent(params);
        this._createPrefsWidget();
    },

    _createPrefsWidget: function() {
        let box;
        let notebook = new Gtk.Notebook({ scrollable: true, height_request: 480, width_request: 640 });
        this.add(notebook, false, false, 0);
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, border_width: 10 });
        for (i in SETTINGS_GLOBAL) {
            box.pack_start(this._createWidget(SETTINGS_GLOBAL[i]), false, false, 5);
        }
        notebook.append_page(box, new Gtk.Label({ label: _("Generally") }));
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, border_width: 10 });
        for (i in SETTINGS_DISPLAY) {
            box.pack_start(this._createWidget(SETTINGS_DISPLAY[i]), false, false, 5);
        }
        notebook.append_page(box, new Gtk.Label({ label: _("Display") }));
    },

    _createWidget: function(args) {
        let box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
        let label = new Gtk.Label({ label: _(args['label']), xalign: 0 });
        let settings = Convenience.getSettings(args['schema']);
        let widget;
        switch (args['type']) {
            case 'combo':
                widget = new Gtk.ComboBoxText();
                for (value in args['values']) {
                    widget.append(value, _(args['values'][value]));
                }
                widget.set_active_id(settings.get_string(args['key']));
                widget.width_request = 200;
                widget.connect('changed', function() {
                    settings.set_string(args['key'], widget.get_active_id());
                });
                settings.connect('changed::'+args['key'], function() {
                    widget.set_active_id(settings.get_string(args['key']));
                });
                break;
            case 'switcher':
                widget = new Gtk.Switch({ active: settings.get_boolean(args['key']) });
                widget.connect('notify::active', function() {
                    settings.set_boolean(args['key'], widget.get_active());
                });
                settings.connect('changed::'+args['key'], function() {
                    widget.set_active(settings.get_boolean(args['key']));
                });
                break;
        }
        if (args['tooltip']) widget.set_tooltip_text(_(args['tooltip']));
        box.pack_start(label, true, true, 0);
        box.add(widget);
        return box;
    }
});
