/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;
const Signals = imports.signals;

const Clutter = imports.gi.Clutter;
const GLib = imports.gi.GLib;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;
const Params = imports.misc.params;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    Help Functions
 */
function checkApplication(application) {
    try {
        return Shell.AppSystem.get_default().lookup_setting(application);
    } catch (err) {
        return Shell.AppSystem.get_default().lookup_app(application);
    }
}

function checkExtension(extension) {
    if (GLib.file_test('/home/'+GLib.get_user_name()+'/.local/share/gnome-shell/extensions/'+extension+'/', GLib.FileTest.EXISTS)) return true;
    if (GLib.file_test('/usr/share/gnome-shell/extensions/'+extension+'/', GLib.FileTest.EXISTS)) return true;
    return false;
}

function getSorted(a, b) {
    if (a.toUpperCase() === b.toUpperCase()) return 0;
    if (a.toUpperCase() > b.toUpperCase()) return 1;
    return -1;
}

function getStringCut(text, strLength) {
    strLength = strLength || 50;
    if (text.length <= strLength) return text;
    return text.substr(0, strLength - 3)+'...';
}

function getStringCutOut(text, strLength) {
    strLength = strLength || 50;
    if (text.length <= strLength) return text;
    strLength = Math.round((strLength - 3) / 2);
    return text.substr(0, strLength - 1)+'...'+text.substr(text.length - strLength, strLength);
}

function getStringLineBreak(text, strLength) {
    let start = strLength || 66;
    for (let i = start; i > start - 50; i--) {
        if (text.substr(i, 1) === (' ' || '-')) {
            text = text.substr(0, i + 1)+'\n'+text.substr(i + 1, text.length - i);
            i = start = i + 55;
            if (i > text.length) break;
        }
    }
    return text;
}

function PopupSwitchMenuItemUpdate(item) {
    let text = item.label.text;
    if (item._hbox) {
        item._hbox.remove_actor(item._statusBin);
        item._hbox.remove_actor(item.label);
        item.removeActor(item._hbox);
        item.label = new St.Label({ text: text });
        item.addActor(item.label);
        item.addActor(item._statusBin, { expand: true, span: -1, align: St.Align.END });
        delete item._hbox;
    } else {
        item.removeActor(item.label);
        item.removeActor(item._statusBin);
        item._hbox = new St.BoxLayout({ vertical: false, style_class: 'paneltools-popupmenu-box' });
        item.addActor(item._hbox, { expand: true });
        item.label = new St.Label({ text: text, y_align: Clutter.ActorAlign.CENTER });
        item._hbox.add(item.label);
        item._hbox.add(new St.Bin(), { expand: true });
        item._hbox.add(item._statusBin, { align: St.Align.END });
    }
}

/*
    Panel
 */
const PANEL_BOX = {
    left:   Main.panel._leftBox,
    center: Main.panel._centerBox,
    right:  Main.panel._rightBox
};

function addToStatusArea(role, indicator, position, box) {
    if (Convenience.checkGSV(3, 4)) {
        if (Main.panel._statusArea[role]) throw new Error(_("Extension point conflict: there is already a status indicator for role '%s'").format(role));
        if (!(indicator instanceof PanelMenu.Button)) throw new TypeError(_("Status indicator must be an instance of PanelMenu.Button"));
        PANEL_BOX[box || 'right'].insert_child_at_index(indicator.actor, position || 0);
        if (indicator.menu) Main.panel._menus.addMenu(indicator.menu);
        Main.panel._statusArea[role] = indicator;
        indicator.actor._rolePosition = position;
        let destroyId = indicator.connect('destroy', Lang.bind(Main.panel, function(emitter) {
            delete Main.panel._statusArea[role];
            emitter.disconnect(destroyId);
        }));
    } else {
        Main.panel.addToStatusArea(role, indicator, position, box);
    }
    return indicator;
}

function getChildren(box) {
    return PANEL_BOX[box].get_children().length;
}

/*
    PanelMenu
 */
const PANELMENU_BUTTON_OFF = 0;
const PANELMENU_BUTTON_ICON_AND_TEXT = 1;
const PANELMENU_BUTTON_ICON = 2;
const PANELMENU_BUTTON_TEXT = 3;

const Button = new Lang.Class({
    Name: 'Button',
    Extends: PanelMenu.Button,

    _init: function(menuAlignment, nameText, params) {
        params = Params.parse(params, { icon_name: null, text: null, dontCreateMenu: false }, true);
        if (menuAlignment !== 0 && menuAlignment !== 1) {
            menuAlignment = menuAlignment || 0.25;
            if (Clutter.get_default_text_direction() === Clutter.TextDirection.RTL) menuAlignment = 1.0 - menuAlignment;
        }
        this.parent(menuAlignment, nameText || '', params.dontCreateMenu);
        this.actor.add_style_class_name('panel-status-button');
        if (!this.accessible_name) this.accessible_name = nameText;
        if (params.dontCreateMenu && (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6))) this.menu = new PopupDummyMenu(this.actor);
        this._hbox = new St.BoxLayout({ vertical: false, style_class: 'paneltools-panelmenu-box' });
        this.actor.add_child(this._hbox);
        if (params.icon_name) {
            this.icon = new St.Icon({ icon_name: params.icon_name, style_class: 'system-status-icon' });
            this._hbox.add(this.icon);
        }
        if (params.text) {
            this.label = new St.Label({ text: params.text, y_align: Clutter.ActorAlign.CENTER });
            this._hbox.add(this.label);
            this.actor.label_actor = this.label;
        }
        if (!params.dontCreateMenu && PopupMenu.arrowIcon) this._hbox.add(PopupMenu.arrowIcon(St.Side.BOTTOM));
        if (!params.dontCreateMenu && PopupMenu.unicodeArrow) this._hbox.add(PopupMenu.unicodeArrow(St.Side.BOTTOM));
    },

    updateButton: function(showButton) {
        this.actor.visible = showButton !== PANELMENU_BUTTON_OFF;
        if (this.actor.visible) {
            if (this.icon) this.icon.visible = showButton === PANELMENU_BUTTON_ICON_AND_TEXT || showButton === PANELMENU_BUTTON_ICON;
            if (this.label) this.label.visible = showButton === PANELMENU_BUTTON_ICON_AND_TEXT || showButton === PANELMENU_BUTTON_TEXT;
        }
    }
});

// Alternate SystemIndicator for GNOME Shell 3.4.x, 3.6.x and 3.8.x
const SystemIndicator = new Lang.Class({
    Name: 'SystemIndicator',

    _init: function() {
        this.indicators = new St.BoxLayout({ style_class: 'paneltools-system-indicator' });
        this.indicators.hide();
        this.menu = new PopupMenu.PopupMenuSection();
    },

    _addIndicator: function() {
        let icon = new St.Icon({ style_class: 'system-status-icon' });
        this.indicators.add(icon);
        this.indicators.show();
        return icon;
    }
});

/*
    PopupMenu
 */
const KEY_MENUS_HAVE_ICONS = 'menus-have-icons';
const KEY_MENUS_ICON_SIZE = 'menus-icon-size';

const POPUPMENU_MENUS_ICON_SIZE = 18;
const POPUPMENU_MENUS_HAVE_ICONS = true;

// PanelManager@dschaerf required to changed the settings for icon size and visible
const PANELMANAGER_SETTINGS_SCHEMA = 'de.dschaerf.extensions.panelManager';

const PopupMenuItem = new Lang.Class({
    Name: 'PopupMenuItem',
    Extends: PopupMenu.PopupBaseMenuItem,

    _init: function (text, wantIcon, params) {
        this.parent(params);
        this._hbox = new St.BoxLayout({ vertical: false, style_class: 'paneltools-popupmenu-box' });
        this.addActor ? this.addActor(this._hbox, { expand: true }) : this.actor.add(this._hbox, { expand: true });
        if (wantIcon) {
            this.icon = this._application ? new St.Bin({ style_class: 'popup-menu-icon' }) : new St.Icon({ style_class: 'popup-menu-icon' });
            this._hbox.add(this.icon);
            this._settings = Convenience.getSettings(PANELMANAGER_SETTINGS_SCHEMA, true);
            if (this._settings) this._connectSettings = [
                this._settings.connect('changed::'+KEY_MENUS_HAVE_ICONS, Lang.bind(this, this._setIconVisible)),
                this._settings.connect('changed::'+KEY_MENUS_ICON_SIZE, Lang.bind(this, this._setIconSize))
            ];
            this._setIconSize();
            this._setIconVisible();
        }
        this.label = new St.Label({ text: text, y_align: Clutter.ActorAlign.CENTER });
        this.actor.label_actor = this.label;
        this._hbox.add(this.label);
        this._hbox.add(new St.Bin(), { expand: true });
        this.status = new St.Label({ text: '', style_class: 'paneltools-popupmenu-inactive-menu-item', y_align: Clutter.ActorAlign.CENTER });
        this._hbox.add(this.status, { align: St.Align.END });
        if (this._connectSettings) this.actor.connect('destroy', Lang.bind(this, function() {
            for each(let connection in this._connectSettings) {
                this._settings.disconnect(connection);
            }
        }));
    },

    _setIconSize: function() {
        if (this._application) {
            this.icon.child = this._application.create_icon_texture(this._settings ? this._settings.get_int(KEY_MENUS_ICON_SIZE) : POPUPMENU_MENUS_ICON_SIZE);
        } else {
            this.icon.icon_size = this._settings ? this._settings.get_int(KEY_MENUS_ICON_SIZE) : POPUPMENU_MENUS_ICON_SIZE;
        }
    },

    _setIconVisible: function() {
        this.icon.visible = this._settings ? this._settings.get_boolean(KEY_MENUS_HAVE_ICONS) : POPUPMENU_MENUS_HAVE_ICONS;
    },

    addButton: function(iconName, callback) {
        this.button = new St.Button({ style_class: 'paneltools-popupmenu-button' });
        this.button.child = new St.Icon({ icon_name: iconName });
        this._hbox.add(this.button, { align: St.Align.END });
        this.button.connect('clicked', Lang.bind(this, function(event) {
            callback(event);
        }));
    },

    setIconName: function(iconName) {
        this.icon.icon_name = iconName;
        if (St.IconType && iconName.indexOf('-symbolic') === -1) this.icon.icon_type = St.IconType.FULLCOLOR;
    },

    setSelected: function(value) {
        if (this.setOrnament) this.setOrnament(value ? PopupMenu.Ornament.DOT : PopupMenu.Ornament.NONE);
        if (this.setShowDot) this.setShowDot(value);
    },

    setStatus: function(text) {
        this.status.text = text;
    }
});

const PopupActionMenuItem = new Lang.Class({
    Name: 'PopupActionMenuItem',
    Extends: PopupMenu.PopupBaseMenuItem,

    _init: function (params) {
        params = Params.parse(params, { span: 1 }, true);
        this.parent({ reactive: false });
        this._hbox = new St.BoxLayout({ style_class: 'paneltools-popupmenu-box' });
        this.addActor ? this.addActor(this._hbox, { expand: true, span: params.span }) : this.actor.add(this._hbox, { expand: true });
    },

    addAction: function(iconName, accessibleName) {
        let action = new St.Button({ reactive: true, can_focus: true, track_hover: true, accessible_name: accessibleName, style_class: 'paneltools-popupmenu-action' });
        action.child = new St.Icon({ icon_name: iconName });
        this._hbox.add(action, { expand: true, x_fill: false });
        return action;
    }
});

const PopupApplicationMenuItem = new Lang.Class({
    Name: 'PopupApplicationMenuItem',
    Extends: PopupMenuItem,

    _init: function(app, wantIcon, params) {
        this._application = app;
        this.parent(app.get_name(), wantIcon, params);
        if (wantIcon) {
            this._setIconSize();
            this._setIconVisible();
        }
        this.connect('activate', Lang.bind(this, function() {
            Main.overview.hide();
            app.activate(-1);
        }));
    }
});

if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
/*
    Only for GNOME Shell 3.4.x and 3.6.x for PanelMenu.Button without menu
*/
const PopupDummyMenu = new Lang.Class({
    Name: 'PopupDummyMenu',

    _init: function(sourceActor) {
        this.sourceActor = sourceActor;
        this.actor = sourceActor;
        this.actor._delegate = this;
    },

    isChildMenu: function() {
        return false;
    },

    open: function() {
        this.emit('open-state-changed', true);
    },

    close: function() {
        this.emit('open-state-changed', false);
    },

    toggle: function() {},

    destroy: function() {
        this.emit('destroy');
    },
});
Signals.addSignalMethods(PopupDummyMenu.prototype);
}

const PopupSubMenuMenuItem = new Lang.Class({
    Name: 'PopupSubMenuMenuItem',
    Extends: PopupMenu.PopupSubMenuMenuItem,

    _init: function(text, wantIcon, params) {
        params = Params.parse(params, { settings: true, style_class: 'popup-menu-icon', span: 1 }, true);
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
            this.parent(text);
            this.removeActor(this.label);
            this.removeActor(this._triangle);
            this._hbox = new St.BoxLayout({ vertical: false, style_class: 'paneltools-popupmenu-box' });
            this.addActor(this._hbox, { expand: true, span: params.span });
            if (wantIcon) {
                this.icon = new St.Icon({ style_class: params.style_class });
                this._hbox.add(this.icon);
            }
            this.label = new St.Label({ text: text, y_align: Clutter.ActorAlign.CENTER });
            this.actor.label_actor = this.label;
            this._hbox.add(this.label);
            this._hbox.add(new St.Bin(), { expand: true });
            this.status = new St.Label({ text: '', style_class: 'paneltools-popupmenu-inactive-menu-item', y_align: Clutter.ActorAlign.CENTER });
            this._hbox.add(this.status, { align: St.Align.END });
            this._triangle = new St.Label({ text: '\u25B8', y_align: Clutter.ActorAlign.CENTER });
            this._hbox.add(this._triangle, { align: St.Align.END });
        } else {
            this.parent(text, wantIcon);
        }
        if (wantIcon && params.settings) {
            this._settings = Convenience.getSettings(PANELMANAGER_SETTINGS_SCHEMA, true);
            if (this._settings) this._connectSettings = [
                this._settings.connect('changed::'+KEY_MENUS_HAVE_ICONS, Lang.bind(this, this._setIconVisible)),
                this._settings.connect('changed::'+KEY_MENUS_ICON_SIZE, Lang.bind(this, this._setIconSize))
            ];
            this._setIconSize();
            this._setIconVisible();
        }
        if (this._connectSettings) this.actor.connect('destroy', Lang.bind(this, function() {
            for each(let connection in this._connectSettings) {
                this._settings.disconnect(connection);
            }
        }));
    },

    _setIconSize: function() {
        this.icon.icon_size = this._settings ? this._settings.get_int(KEY_MENUS_ICON_SIZE) : POPUPMENU_MENUS_ICON_SIZE;
    },

    _setIconVisible: function() {
        this.icon.visible = this._settings ? this._settings.get_boolean(KEY_MENUS_HAVE_ICONS) : POPUPMENU_MENUS_HAVE_ICONS;
    },

    setIconName: function(iconName) {
        this.icon.icon_name = iconName;
        if (St.IconType && iconName.indexOf('-symbolic') === -1) this.icon.icon_type = St.IconType.FULLCOLOR;
    },

    setStatus: function(text) {
        this.status.text = text;
    }
});

/*
    Separator
 */
const VerticalSeparator = new Lang.Class({
    Name: 'VerticalSeparator',

    _init: function() {
        this.actor = new St.DrawingArea({ style_class: 'paneltools-vertical-separator', pseudo_class: 'highlighted' });
        this.actor.connect('repaint', Lang.bind(this, this._onRepaint));
    },

    _onRepaint: function(area) {
        let cr = area.get_context();
        let themeNode = area.get_theme_node();
        let [width, height] = area.get_surface_size();
        let stippleColor = themeNode.get_color('-stipple-color');
        let stippleWidth = themeNode.get_length('-stipple-width');
        let x = Math.floor(width / 2) + 0.5;
        cr.moveTo(x, 0);
        cr.lineTo(x, height);
        Clutter.cairo_set_source_color(cr, stippleColor);
        cr.setDash([1, 3], 1);
        cr.setLineWidth(stippleWidth);
        cr.stroke();
        cr.$dispose();
    }
});
